@@ -1,7 +1,7 @@
 /*
 **  OSSP uuid - Universally Unique Identifier
-**  Copyright (c) 2004-2006 Ralf S. Engelschall <rse@engelschall.com>
-**  Copyright (c) 2004-2006 The OSSP Project <http://www.ossp.org/>
+**  Copyright (c) 2004-2007 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 2004-2007 The OSSP Project <http://www.ossp.org/>
 **
 **  This file is part of OSSP uuid, a library for the generation
 **  of UUIDs which can found at http://www.ossp.org/pkg/lib/uuid/
@@ -243,12 +243,13 @@ void uuid::import(const void *bin)
     return;
 }
 
-/*  method: import string representation */
+/*  method: import string or single integer value representation */
 void uuid::import(const char *str)
 {
     uuid_rc_t rc;
     if ((rc = uuid_import(ctx, UUID_FMT_STR, str, UUID_LEN_STR)) != UUID_RC_OK)
-        throw uuid_error_t(rc);
+        if ((rc = uuid_import(ctx, UUID_FMT_SIV, str, UUID_LEN_SIV)) != UUID_RC_OK)
+            throw uuid_error_t(rc);
     return;
 }
 
@@ -272,6 +273,16 @@ char *uuid::string(void)
     return str;
 }
 
+/*  method: export single integer value representation */
+char *uuid::integer(void)
+{
+    uuid_rc_t rc;
+    char *str = NULL;
+    if ((rc = uuid_export(ctx, UUID_FMT_SIV, (void **)&str, NULL)) != UUID_RC_OK)
+        throw uuid_error_t(rc);
+    return str;
+}
+
 /*  method: export textual summary representation */
 char *uuid::summary(void)
 {
